﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;

namespace Kevin.WebForm.Sample
{
    /// <summary>
    /// 视图生成控制器
    /// </summary>
    internal class Controller : Page
    {
        /// <summary>
        /// 覆盖控件生成
        /// </summary>
        /// <param name="control"></param>
        public override void VerifyRenderingInServerForm(Control control)
        {

        }
    }

    /// <summary>
    /// 视图生成引擎
    /// </summary>
    public class ViewEngine<T> where T : UserControl, IDisposable
    {
        private Controller _controller;

        public ViewEngine()
        {
            _controller = new Controller();
        }

        /// <summary>
        /// 加载控件
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public T LoadViewControl(string path)
        {
            _controller.EnableViewState = false;
            return (T)_controller.LoadControl(path);
        }

        /// <summary>
        /// 生成视图
        /// </summary>
        /// <param name="control">控件类</param>
        /// <returns>返回控件HTML</returns>
        public string RenderView(T control)
        {
            try
            {
                _controller.Controls.Add(control);
                StringWriter output = new System.IO.StringWriter();
                HttpContext.Current.Server.Execute(_controller, output, false);
                _controller.Controls.Clear();
                return output.ToString();
            }
            catch { return string.Empty; }
        }

        /// <summary>
        /// 施放资源
        /// </summary>
        public void Dispose()
        {
            _controller.Dispose();
        }
    }
}